@@ -5,15 +5,16 @@
     :contact: code@inmanta.com
     :license: Inmanta EULA
 """
-
+import json
 import logging
 from pprint import pformat
 from typing import Any, Collection, Dict, List, Optional, Union
 from uuid import UUID
 
+from inmanta_lsm.diagnose.model import FullDiagnosis
+
 from pytest_inmanta_lsm import remote_orchestrator as r_orchestrator
 from pytest_inmanta_lsm.exceptions import VersionExceededError, VersionMismatchError
-from pytest_inmanta_lsm.failed_resources_logs import FailedResourcesLogs
 from pytest_inmanta_lsm.wait_for_state import State, WaitForState
 
 LOGGER = logging.getLogger(__name__)
@@ -40,6 +41,8 @@ class ManagedServiceInstance:
 
     ALL_BAD_STATES = list(set(CREATE_FLOW_BAD_STATES + UPDATE_FLOW_BAD_STATES + DELETE_FLOW_BAD_STATES))
 
+    DEFAULT_TIMEOUT = 600
+
     def __init__(
         self,
         remote_orchestrator: "r_orchestrator.RemoteOrchestrator",
@@ -70,6 +73,7 @@ class ManagedServiceInstance:
         version: Optional[int] = None,
         versions: Optional[Collection[int]] = None,
         bad_states: Collection[str] = CREATE_FLOW_BAD_STATES,
+        timeout: int = DEFAULT_TIMEOUT,
     ) -> None:
         """
         Create the service instance and wait for it to go into `wait_for_state` or one of `wait_for_states` and
@@ -82,6 +86,7 @@ class ManagedServiceInstance:
         :param version: the target state should have this version number, defaults to None
         :param versions: the target state should have one of those version numbers, defaults to None
         :param bad_states: stop waiting and fail if any of these states are reached, defaults to CREATE_FLOW_BAD_STATES
+        :param timeout: how long can we wait for service to achieve given state (in seconds)
         :raises BadStateError: If the instance went into a bad state
         :raises TimeoutError: If the timeout is reached while waiting for the desired state(s)
         :raises ValueError: If both of state and states are set
@@ -122,6 +127,7 @@ class ManagedServiceInstance:
             version=version,
             versions=versions,
             bad_states=bad_states,
+            timeout=timeout,
         )
 
     def update(
@@ -133,6 +139,7 @@ class ManagedServiceInstance:
         current_version: Optional[int] = None,
         attribute_updates: Dict[str, Union[str, int]] = {},
         bad_states: Collection[str] = UPDATE_FLOW_BAD_STATES,
+        timeout: int = DEFAULT_TIMEOUT,
     ) -> None:
         """
         Update the service instance with the given `attributes_updates` and wait for it to go into `wait_for_state` or one
@@ -146,6 +153,7 @@ class ManagedServiceInstance:
         :param current_version: current version, defaults to None
         :param attribute_updates: dictionary containing the key(s) and value(s) to be updates, defaults to {}
         :param bad_states: stop waiting and fail if any of these states are reached, defaults to UPDATE_FLOW_BAD_STATES
+        :param timeout: how long can we wait for service to achieve given state (in seconds)
         :raises BadStateError: If the instance went into a bad state
         :raises TimeoutError: If the timeout is reached while waiting for the desired state(s)
         :raises ValueError: If both of state and states are set
@@ -179,6 +187,7 @@ class ManagedServiceInstance:
             versions=new_versions,
             bad_states=bad_states,
             start_version=current_version,
+            timeout=timeout,
         )
 
     def delete(
@@ -189,6 +198,7 @@ class ManagedServiceInstance:
         versions: Optional[Collection[int]] = None,
         current_version: Optional[int] = None,
         bad_states: Collection[str] = DELETE_FLOW_BAD_STATES,
+        timeout: int = DEFAULT_TIMEOUT,
     ) -> None:
         """
         Delete the service instance and wait for it to go into `wait_for_state` or one of `wait_for_states` and
@@ -201,6 +211,7 @@ class ManagedServiceInstance:
         :param new_versions: the target state should have one of those version numbers, defaults to None
         :param current_version: current version, defaults to None
         :param bad_states: stop waiting and fail if any of these states are reached, defaults to UPDATE_FLOW_BAD_STATES
+        :param timeout: how long can we wait for service to achieve given state (in seconds)
         :raises BadStateError: If the instance went into a bad state
         :raises TimeoutError: If the timeout is reached while waiting for the desired state(s)
         :raises ValueError: If both of state and states are set
@@ -232,6 +243,7 @@ class ManagedServiceInstance:
             versions=versions,
             bad_states=bad_states,
             start_version=current_version,
+            timeout=timeout,
         )
 
     def get_state(
@@ -276,7 +288,7 @@ class ManagedServiceInstance:
         states: Optional[Collection[str]] = None,
         version: Optional[int] = None,
         versions: Optional[Collection[int]] = None,
-        timeout: int = 600,
+        timeout: int = DEFAULT_TIMEOUT,
         bad_states: Collection[str] = ALL_BAD_STATES,
         start_version: Optional[int] = None,
     ) -> None:
@@ -338,22 +350,19 @@ class ManagedServiceInstance:
                 return False
             return current_state.version == start_version
 
-        def get_bad_state_error(current_state) -> Any:
-            validation_failure_msg = self.remote_orchestrator.get_validation_failure_message(
-                service_entity_name=self.service_entity_name,
-                service_instance_id=self.instance_id,
+        def get_bad_state_error(current_state: State) -> FullDiagnosis:
+            result = self.remote_orchestrator.client.lsm_services_diagnose(
+                tid=self.remote_orchestrator.environment,
+                service_entity=self.service_entity_name,
+                service_id=self.instance_id,
+                version=current_state.version,
             )
-            if validation_failure_msg:
-                return validation_failure_msg
-
-            LOGGER.info("No validation failure message, getting failed resource logs")
-
-            # No validation failure message, so getting failed resource logs
-            failed_resource_logs = FailedResourcesLogs(
-                self.remote_orchestrator.client,
-                self.remote_orchestrator.environment,
+            assert result.code == 200, (
+                f"Wrong response code while trying to get the service diagnostic, got {result.code} (expected 200):\n"
+                f"{json.dumps(result.result or {}, indent=4)}"
             )
-            return failed_resource_logs.get()
+
+            return FullDiagnosis(**result.result["data"])
 
         wait_for_obj = WaitForState(
             "Instance lifecycle",
